using System;
using System.Collections.Concurrent;
using System.IO;
using System.Threading.Tasks;

using Microsoft.Extensions.FileProviders;

namespace NSeguin.Dev.Web.HeroIcons;

internal readonly record struct HeroIconParameters(HeroIconKind Kind, HeroIconVariant Variant)
{
    private const string PathBase = "lib/heroicons";

    private static readonly ConcurrentDictionary<HeroIconParameters, string> SvgCache = new();
    private static ManifestEmbeddedFileProvider FileProvider = new(typeof(HeroIcon).Assembly);

    public Task<string> GetSvgAsync()
    {
        if (SvgCache.TryGetValue(this, out var svg))
        {
            return Task.FromResult(svg);
        }

        return LoadSvgAsync();
    }

    private static string GetPath(HeroIconVariant variant, HeroIconKind kind)
    {
        var pathSuffix = variant switch
        {
            HeroIconVariant.Solid => "24/solid",
            HeroIconVariant.Outline => "24/outline",
            HeroIconVariant.Small => "20/solid",
            HeroIconVariant.ExtraSmall => "16/solid",
            _ => throw new ArgumentOutOfRangeException(nameof(variant))
        };

        return Path.Combine(PathBase, pathSuffix, kind.FileName);
    }

    private string GetPath() => GetPath(Variant, Kind);

    private async Task<string> LoadSvgAsync()
    {
        var path = GetPath();
        await using var fileStream = FileProvider.GetFileInfo(path).CreateReadStream();
        using var reader = new StreamReader(fileStream);
        var svg = await reader.ReadToEndAsync();
        SvgCache.TryAdd(this, svg);
        return svg;
    }
}
